"""
Utils

-----

This script is full of general functions used by the other scripts in this package.
"""
import os
import shutil
import subprocess
import sys
from os.path import exists
import struct
import urllib.request
import dateutil.parser
import requests
import tarfile
import zipfile


class Colors:
    W = '\033[0m'
    R = '\033[31m'
    G = '\033[32m'
    O = '\033[33m'
    B = '\033[34m'
    P = '\033[35m'
    C = '\033[36m'
    BOLD = '\033[1m'
    BLINK = '\033[5m'
    R_BOLD = '\033[21m'
    R_BLINK = '\033[25m'
    RM = '\033[0m'
    t = B
    pre = f"{BOLD}{G}[{P}MSG{G}] {RM}{t}"


user_agent = {"User-Agent": "ashduino101/monosodium-glutamate"}


class UnsupportedPlatform(Exception):
    def __init__(self,
                 platform,
                 message="Your platform is unsupported",
                 supported="Valid platforms: Windows (win32), Mac OS X (darwin), Linux (linux/linux2)"):
        self.message = message
        self.supported = supported
        self.platform = platform
        super().__init__(self.message)

    def __str__(self):
        return f"{self.message}: '{self.platform}'. {self.supported}"


def readable_date(date):
    d = dateutil.parser.parse(date)
    return d.strftime('%b %d, %Y, at %H:%M:%S')


def datetime_date(date):
    return dateutil.parser.parse(date)


def get_file_from_md5(md5, ext, api):
    base = "https://static1." + api.replace("https://", "")
    return f"{base}/data/{md5[0:2]}/{md5[2:4]}/{md5}.{ext}"


def ruffle_setup(platform):
    print(Colors.pre + "Starting first time Ruffle setup...")
    print(Colors.pre + "Fetching releases...", end=" ")
    releases = requests.get("https://api.github.com/repos/ruffle-rs/ruffle/releases").json()
    print(Colors.C + "done.")
    latest = releases[0]["assets_url"]
    assets = requests.get(latest).json()
    linux = ""
    macos = ""
    win32 = ""
    win64 = ""
    print(Colors.pre + "Finding platform-specific files...", end=" ")
    for asset in assets:
        name = asset["name"]
        file = asset["browser_download_url"]
        if name.endswith("linux-x86_64.tar.gz"):
            linux = file, name
        elif name.endswith("macos-universal.tar.gz"):
            macos = file, name
        elif name.endswith("windows-x86_32.zip"):
            win32 = file, name
        elif name.endswith("windows-x86_64.zip"):
            win64 = file, name
    print(Colors.C + "done.")
    is64bit = struct.calcsize('P') * 8 == 64
    print(Colors.pre + ("Platform is 64-bit." if is64bit else "Platform is 32-bit."))
    print(Colors.pre + "Detecting platform...", end=" ")
    if platform == "linux" or platform == "linux2":
        dl = linux[0]
        n = linux[1]
    elif platform == "win32" and not is64bit:
        dl = win32[0]
        n = win32[1]
    elif platform == "win32" and is64bit:
        dl = win64[0]
        n = win64[1]
    elif platform == "darwin":
        dl = macos[0]
        n = macos[1]
    else:
        raise UnsupportedPlatform(platform)
    print(Colors.C + f"done. Platform is {platform}")
    if not exists("./.e6"):
        os.mkdir("./.e6")
        print(Colors.pre + "Created ./.e6/.")
    if not exists("./.e6/.cache"):
        os.mkdir("./.e6/.cache")
        print(Colors.pre + "Created ./.e6/.cache/.")
    if not exists("./.e6/ruffle"):
        os.mkdir("./.e6/ruffle")
        print(Colors.pre + "Created ./.e6/ruffle/.")
    urllib.request.urlretrieve(dl, f"./.e6/.cache/{n}")
    print(Colors.pre + "Retrieved Ruffle archive.")
    if platform == "win32":
        with zipfile.ZipFile(f"./.e6/.cache/{n}") as z:
            z.extractall("./.e6/ruffle/")
    else:
        with tarfile.open(f"./.e6/.cache/{n}") as t:
            t.extractall("./.e6/ruffle")
    print(Colors.pre + "Deleting cached archives...", end=" ")
    shutil.rmtree("./.e6/.cache/")
    print(Colors.C + "done.")
    print(Colors.pre + "Extracted binary file to ./.e6/ruffle/")
    print(Colors.pre + "Finished setting up Ruffle.")


def ruffle(url):
    if not exists(f"./.e6/ruffle/{'ruffle.exe' if sys.platform == 'win32' else 'ruffle'}"):
        ruffle_setup(sys.platform)
    subprocess.call([f"./.e6/ruffle/{'ruffle.exe' if sys.platform == 'win32' else 'ruffle'}", url])
